"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_utils_1 = require("../path.utils");
const rmrf_1 = __importDefault(require("./rmrf"));
/**
 * @internal
 */
function cleanTargets(targets, compilerOptions, basePath = process.cwd()) {
    protectSensitiveFolders(targets, compilerOptions, basePath);
    if (Array.isArray(targets)) {
        for (let target of targets) {
            target = path_utils_1.ensureAbsolutePath(target, basePath);
            console.log('Cleaning:', path_utils_1.relativePath(target));
            rmrf_1.default(target);
        }
    }
    else {
        const { outDir, outFile, declarationDir } = compilerOptions;
        if (targets.outDir && outDir) {
            console.log('Cleaning outDir:', path_utils_1.relativePath(outDir));
            rmrf_1.default(outDir);
        }
        if (targets.declarationDir && declarationDir) {
            console.log('Cleaning declarationDir:', path_utils_1.relativePath(declarationDir));
            rmrf_1.default(declarationDir);
        }
        if (targets.outFile && outFile) {
            console.log('Cleaning outFile:', path_utils_1.relativePath(outFile));
            rmrf_1.default(outFile);
        }
    }
}
exports.default = cleanTargets;
/**
 * @internal
 */
function protectSensitiveFolders(targets, compilerOptions, basePath) {
    const cwd = process.cwd();
    let protectedDirs = [cwd, ...path_utils_1.parentPaths(cwd)];
    if (cwd !== basePath) {
        protectedDirs.push(basePath, ...path_utils_1.parentPaths(basePath));
    }
    // `compilerOptions` properties returns unix separators in windows paths so we must normalize
    const rootDir = path_utils_1.normalizePath(compilerOptions.rootDir);
    if (rootDir && rootDir !== cwd && rootDir !== basePath) {
        protectedDirs.push(rootDir, ...path_utils_1.parentPaths(rootDir));
    }
    // Dedupe
    protectedDirs = [...new Set(protectedDirs)];
    const check = (target) => {
        for (const dir of protectedDirs) {
            if (target === dir)
                throw Error(`You cannot delete ${target}`);
        }
    };
    if (Array.isArray(targets)) {
        for (let target of targets) {
            target = path_utils_1.ensureAbsolutePath(target, basePath);
            check(target);
        }
    }
    else {
        for (const target of Object.keys(targets)) {
            if (compilerOptions[target]) {
                // We can have outDir/declarationDir to be the same as rootDir
                check(compilerOptions[target]);
            }
        }
    }
}
