"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
/**
 * Inspired from {@link https://github.com/isaacs/rimraf}
 * @internal
 */
function rmrf(path) {
    // TODO: rely on ENOENT instead
    // https://nodejs.org/dist/latest/docs/api/fs.html#fs_fs_exists_path_callback
    if (!fs_1.existsSync(path))
        return;
    if (tryIsDirectory(path))
        deleteRecursive(path);
    else
        tryDeleteFile(path);
    function deleteRecursive(dirPath) {
        const names = fs_1.readdirSync(dirPath);
        for (const name of names) {
            const path_ = path_1.join(dirPath, name);
            if (tryIsDirectory(path_))
                deleteRecursive(path_);
            else
                tryDeleteFile(path_);
        }
        tryDeleteEmptyDir(dirPath);
    }
}
exports.default = rmrf;
/**
 * @internal
 */
function tryIsDirectory(path) {
    const stats = fsSyncRetry(fs_1.lstatSync, path, ['EPERM'], 2);
    return stats.isDirectory();
}
/**
 * @internal
 */
function tryDeleteFile(filePath) {
    fsSyncRetry(fs_1.unlinkSync, filePath, ['EBUSY', 'EPERM', 'ENOTEMPTY']);
}
/**
 * @internal
 */
function tryDeleteEmptyDir(dirPath) {
    fsSyncRetry(fs_1.rmdirSync, dirPath, ['EBUSY', 'EPERM', 'ENOTEMPTY']);
}
/**
 * @internal
 */
function fsSyncRetry(fsSyncFn, path, errorCodes, tries = 13) {
    let round = 1;
    do {
        if (round > 4) {
            console.log(`Try ${fsSyncFn.name} on ${path_1.relative(process.cwd(), path)} for the ${round}th time (out of ${tries})`);
        }
        try {
            return fsSyncFn(path);
        }
        catch (err) {
            if (!errorCodes.includes(err.code) || round > tries)
                throw err;
            fixWindowsEPERM(path);
            pause(round * 100);
            round++;
        }
    } while (true);
    function pause(ms) {
        const until = Date.now() + ms;
        // tslint:disable-next-line: no-empty
        while (Date.now() < until) { }
    }
    function fixWindowsEPERM(path_) {
        if (process.platform === 'win32')
            fs_1.chmodSync(path_, 0o666);
    }
}
